// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#include "animation_cycle.h"

AnimationCycle::AnimationCycle(Gtk::TreeModel::iterator tIter) :
   iter(tIter), blendingFactor_(0.0), weight_(1.0)
{
	
  blendingAd_ = new Gtk::Adjustment(blendingFactor_, 0.0001, 1.0, 0.01, 0.5, 0.0);
  blendingAd_->signal_value_changed().connect(
      sigc::mem_fun(*this, &AnimationCycle::on_blending_changed));
}

AnimationCycle::~AnimationCycle() {
  if(blendingAd_) {
    delete blendingAd_;
  }
}


void AnimationCycle::on_blending_changed() {
  blendingFactor_ = blendingAd_->get_value();
  blending_changed(iter);
}

void AnimationCycle::setBlending(float blending) {
  if(blending > 1.0) {
    blending = 1.0;
  } else if (blending < 0.00001) {
    blending = 0.00001;
  }
  blendingFactor_ = blending;
  blendingAd_->set_value(blending);
  blending_changed(iter);
}

Gtk::HScale *AnimationCycle::createScaleWidget() {
  Gtk::HScale *t = new Gtk::HScale(*blendingAd_);
  t->set_digits(2);
  t->property_draw_value().set_value(false);
  return t;
}

Gtk::SpinButton *AnimationCycle::createSpinButton() {
  Gtk::SpinButton *t = new Gtk::SpinButton(*blendingAd_, 1.0, 2);
  t->set_numeric(true);
  return t;
}
