// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#ifndef _EF_ANIMATION_CYCLE_H_
#define _EF_ANIMATION_CYCLE_H_

#include <gtkmm.h>
#include <glibmm.h>
#include <sigc++/object.h>
#include <sigc++/object_slot.h>

/**
 * This class takes care of the animation cycle, blending animations and playing them
 *
 *\author Henri Kuuste
 */
class AnimationCycle : virtual public sigc::trackable {
 public:
  /**
   * Constructor
   *
   *\param tIter an iterator over the animation menu GTK tree
   */
  AnimationCycle(Gtk::TreeModel::iterator tIter);

  /**
   * Default destructor
   */
  ~AnimationCycle();

  /**
   * Set the blending factor to a value between 0.0 and 1.0
   *
   *\param blending float between 0.0 and 1.0
   */
  void setBlending(float blending);

  /**
   * Get the blending factor that's currently set
   *\return a value from 0.0 to 1.0
   */
  float getBlending() { return blendingFactor_; }

  /**
   * Callback to call when the blending changed
   */
  void on_blending_changed();

  /**
   * A horizontal bar showing the blending factor of the animation cycle
   */
  Gtk::HScale *createScaleWidget();

  /**
   * A spin button to turn the animation on or off
   */
  Gtk::SpinButton *createSpinButton();

  /**
   * An iterator over the GTK tree view object
   */
  Gtk::TreeModel::iterator iter;

  /**
   * Signal emmited when the blending changes
   */
  sigc::signal1<void,Gtk::TreeIter> blending_changed;

 protected:
  float blendingFactor_;
  float weight_;
  Gtk::Adjustment *blendingAd_;
};

#endif
