// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#include "general.h"
#include <glibmm.h>

std::string fixPathSlashes(const std::string &str) {
  std::string s(str);
  std::string ss = (G_DIR_SEPARATOR_S == "/" ? "\\" : "/");
  std::string::size_type pos;
  while((pos = s.find_first_of(ss)) != std::string::npos) {
    s = s.replace(pos,1,G_DIR_SEPARATOR_S);
  }
  return trim_both(s);
}
std::string trim_right(const std::string &str) {
  std::string s(str);
  return s.erase( s.find_last_not_of(" ") + 1);
}
std::string trim_left(const std::string &str) {
  std::string s(str);
  return s.erase(0, s.find_first_not_of(" "));
}
std::string trim_both(const std::string &str) {
  std::string s(str);
  return trim_right(trim_left(s));
}


// Kai -- basic idea stolen from sear
std::string getConfigFile(){
#ifdef __WIN32__
  std::string path = getenv("USERPROFILE");
  if (path.empty()) {
    const char *homedrive = getenv("HOMEDRIVE");
    const char *homepath = getenv("HOMEPATH");

    if (!homedrive || !homepath) {
        std::cerr << "unable to determine homedir in Win32, using ." << std::endl;
        return ".\\entityforge.cfg";
    }
    path = std::string(homedrive) + std::string(homepath);
  }
  return path + "\\Application Data\\EntityForge\\entityforge.cfg";
#elif __APPLE__
    return getAppSupportDirPath() + "/EntityForge/entityforgerc";
#else
  std::string path = getenv("HOME");
  if (path.empty()) {
    std::cerr << "$HOME not set, using '.' for user settings" << std::endl;
    return "./.entityforgerc";
  }

  return path + "/.entityforgerc";
#endif
}
