// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste
// Changes copyright (C) 2004 Kai Blin

#ifndef _EF_MAINWINDOW_H_
#define _EF_MAINWINDOW_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtkmm.h>
#include <glibmm.h>
#include "model.h"
#include "glView.h"
#include "model_cols.h"
#include "obj.h"
#include "static_object.h"
#include "md3_model.h"
#include <cal3d/cal3d.h>

#include <varconf/Config.h>
#include "general.h"

#include <limits.h>
#include <fstream>
#include <ios>
#include <sstream>
#include <list>
#include <vector>

#include <gtkglmm.h>

#include <sigc++/object_slot.h>
#include <sigc++/bind.h>

#ifdef G_OS_WIN32
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

// FIXME: What is this? Why isn't this  const?
#define HITS_BUFFER_SIZE 512

/** The class responsible for the GUI
 *
 *\author Henri Kuuste
 */

class mainWindow : public sigc::trackable {
public:
  mainWindow();
  ~mainWindow();
	
  Gtk::Window *mainWin;
  Model *calm;

  ///
  enum Selection {
    SUBMESHES,
    JOINTS,
    STATIC_OBJECTS
  };

  ///
  enum RotationMode {
    ARCBALL,
    EULER
  };

  ///
  enum ActionMode {
    NONE,
    ROTATING_OBJECTS
  };

protected:
  void createGUI();
  Gtk::MenuBar *createMenu();
  Gtk::Menu *createFileMenu();
  Gtk::Menu *createEditMenu();
  Gtk::Menu *createViewMenu();
  Gtk::Menu *createOptionsMenu();
  Gtk::Menu *createHelpMenu();
  Gtk::Toolbar *createTopToolbar();
  Gtk::Table *createToolbarTable();
  void createAboutDialog();
  void createControlsHelpDialog();
  void createExecuteActionDialog();
  Gtk::Toolbar *createRightToolbar();
  Gtk::EventBox *createViewport();
  Gtk::TreeView *createTreeView();
  Gtk::HBox *createBottomInfo();
  Gtk::Table *createCyclesInfoTable(Gtk::TreeModel::iterator iter);
  Gtk::Table *createModelInfoTable(Gtk::TreeModel::iterator iter);
  Gtk::Table *createMeshInfoTable(Gtk::TreeModel::iterator iter);
  Gtk::Table *createSubmeshInfoTable(Gtk::TreeModel::iterator iter);
  Gtk::Table *createMaterialInfoTable(Gtk::TreeModel::iterator iter);
  Gtk::Table *createCycleInfoTable(Gtk::TreeModel::iterator iter);
  Gtk::Table *createModelsView();
	
  void on_new();
  void on_open();
  void on_save();
  void on_save_as();
  void on_cal3d_cfg_export();
  void on_recent(Glib::ustring filename);
  void on_import();
  bool loadCFG(const Glib::ustring filename);
  bool loadMd3(const std::string filename);
  bool loadObj(const std::string filename);
  void tree_from_model();
  void refresh();
  void reset_transforms();
  void set_rotMode(RotationMode m, bool elsewhere = false);
  void rotate_scene();
  void on_lod();

  void on_sizeScale();
  void on_timeScale();
  void on_lights();
  void on_select();
  void on_skeleton();
  void on_wireframe();
  void on_backface_culling();
  void on_wire_on_surface();
  void on_background_color();
  void on_selection_color();
  bool on_tree_event(GdkEvent *event);
  void on_tree_selection_changed();
  void on_tree_active_toggled(const Glib::ustring &path_string);
  void on_menu_cw();
  void on_menu_ccw();
  void on_static_object();
  void on_obj();
  void on_md3();
  void on_setup_static_object();
  void on_rotate_static_objects();
  void generic_menu_handler() { std::cout << "Not implemented..." << std::endl; }
  void execute_action_handler(bool read);
  void change_material_handler();
  void blend_in_cycle();
  CalVector get_mouse_pos(double x, double y);
  bool on_viewport_button_press(GdkEventButton *event);
  bool on_viewport_button_release(GdkEventButton *event);
  bool on_viewport_mouse_move(GdkEventMotion *event);
  bool on_main_mouse_move(GdkEventMotion *event);
  void on_main_key_press(GdkEventKey *event);
  void on_main_key_release(GdkEventKey *event);
  void on_cycle_toggle(Gtk::TreeModel::Children::iterator iter);
  void renderGL(double seconds);
  int getPolyCount();
  void setPolyCount();
  void getSelection(double x, double y);
  void processHits(GLint hits, GLuint buffer[]);
  void loadConfig();
  void saveConfig();

  varconf::Config conf;
  bool settingMaterial;
  int mSetMeshId, mSetId;
	
  Gtk::Dialog *aboutDialog;
  Gtk::Dialog *controlsHelpDialog;
  Gtk::Dialog *executeActionDialog;
  Gtk::MenuItem *about;
  Gtk::Label *polyCount;
  Gtk::Label *pcLabel;
  Gtk::Label *fps;
  unsigned int frames;
  double frameSeconds;
	
  Gtk::TreeView *entityTree;
  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection;
  Gtk::TreeModel::iterator lastSelected;

  bool m_select_mode;
  Gtk::ToggleToolButton *select_button;
  int selectedBone;
  bool boneSelected;
  vector<StaticObject*> s_objects;
  int objToRotate;
  ArcBall ball;

  ActionMode aMode;
  RotationMode rotMode;
  Selection selectionFlags;
	
  Gtk::HPaned *treePane;
  Gtk::VPaned *infoPane;
  Gtk::VBox *infoBox;

  Gtk::CheckMenuItem *backfaceCulling, *wireOnSurface, *cw, *ccw, *mode_arcball, *mode_euler;
  Gtk::SpinButton *fadeInDelaySpin, *fadeOutDelaySpin;

  Gtk::EventBox *viewportFrame;
  glView *viewport;
  GLenum frontFace;

  Gtk::HScale *lodScale, *timeScale, *size_scale;

  bool lit, skeleton, wireframe;
  bool wOnSurface, culling;
  double lastRotX, lastRotY;
  double lastTranX, lastTranY;
  CalVector t, r;
  CalVector initT, initR;
  double selColor[3];
	
  Glib::ustring lastPath;
  unsigned int num_recent_files;
  std::list<Glib::ustring> recentFiles;
  void add_recent_file(const Glib::ustring &filename);
  Gtk::Menu *recentFilesMenu;
	
  Gtk::Menu *tMenu;
  Gtk::Statusbar *helpBar;
	
private:
  Glib::ustring filename;
  double lastSeconds;
  ModelColumns cols;
};

#endif // _EF_MAINWINDOW_H_
