// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2004 Kai Blin

#ifndef _EF_MD3_MODEL_H_
#define _EF_MD3_MODEL_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_LIBMD3

extern "C" {
#include <libmd3/structure.h>
#include <libmd3/loader.h>
#include <libmd3/convert.h>
#include <libmd3/mesh.h>
}

#include <cal3d/cal3d.h>
#include <GL/gl.h>

#include "static_object.h"
#include "general.h"
#include "image_loader.h"

/**
 * A class wrapping the functionality of libmd3 for EntityForge
 *
 *\author Kai Blin
 */

class Md3Model : public StaticObject{
public:

  /**
   * Default constructor
   */
  Md3Model();

  /**
   * Contructor taking a Cal3d bone
   *\param bone cal3d bone to attach to
   */
  Md3Model(CalBone *bone);

  /**
   * Default destructor
   */
  virtual ~Md3Model();

  /**
   * Load model from a file
   *
   *\param file std::string containing the filename
   *\return true if everything worked, false otherwise
   */
  bool readFile(std::string file);

  /**
   * Draw one mesh
   *
   *\param mesh mesh to draw
   */
  void drawOneMesh(libmd3_mesh *mesh);

  /**
   * Render the model
   */
  void render();

protected:
  
  /**
   * A pointer to the md3 file
   */
  libmd3_file *m_model_file;

};


#endif //HAVE_LIBMD3

#endif 
